:: GameDataConvert.bat
:: Based on a vanilla Oni (Windows or Mac) GameDataFolder, creates a GameDataFolderX for use with OniX.exe

@echo off
setlocal enableDelayedExpansion

SET HAS_SEP=0
SET RAWSEP_MISSING=0

echo == OniX game data converter v1.1 ==

REM ==========================
echo Inspecting Oni environment...
IF NOT EXIST GameDataTool.exe (
	echo GameDataTool not found. GameDataTool.exe needs to be placed alongside this script.
	PAUSE
	EXIT
)

IF NOT EXIST GameDataFolder\ (
	echo There is no GameDataFolder present here. The folder needs to be placed alongside this script.
	echo If you do not own a copy of Oni, you can download the demo from Oni Mod Depot.
	echo Visit http://mods.oni2.net/ and search for 'demo' to get it.
	PAUSE
	EXIT
)

SET PRIOR_GDFX=0
IF EXIST GameDataFolderX\ (
	REM We are not concerned with other things that might be in GDFX, just the level files
	IF EXIST GameDataFolderX\level*.dat (
		SET PRIOR_GDFX=1
	)
	IF EXIST GameDataFolderX\level*.raw (
		SET PRIOR_GDFX=1
	)
	IF EXIST GameDataFolderX\level*.sep (
		SET PRIOR_GDFX=1
	)
	IF !PRIOR_GDFX!==1 (
		echo Existing GameDataFolderX detected. Your previously-converted game data will be overwritten.
		echo Do you want to remove the previous contents of GameDataFolderX and run the conversion again?
		SET /p CHOICE="(y/n) "
		IF "!CHOICE!"=="y" (
			DEL GameDataFolderX\level*.dat 2> nul
			DEL GameDataFolderX\level*.raw 2> nul
			DEL GameDataFolderX\level*.sep 2> nul
			echo Removed previous conversion products in GameDataFolderX.
		) ELSE (
			EXIT
		)
	)
)

SET PRIOR_CONV=0
FOR %%F IN (GameDataFolder\level*.dat) DO (
	SET FOLDER="GameDataFolder\%%~nF"
	IF EXIST "!FOLDER!" SET PRIOR_CONV=1
	SET LOG_PATH="GameDataFolder\%%~nF.log"
	IF EXIST "!LOG_PATH!" SET PRIOR_CONV=1
)
IF !PRIOR_CONV!==1 (	
	echo Products of an earlier conversion detected in GameDataFolder. Do you want to remove them
	echo and run the conversion again?
	SET /p CHOICE="(y/n) "
	IF "!CHOICE!"=="y" (
		FOR %%F IN (GameDataFolder\level*.dat) DO (
			SET FOLDER="GameDataFolder\%%~nF"
			IF EXIST "!FOLDER!" RMDIR "!FOLDER!" /q /s
			SET LOG_PATH="GameDataFolder\%%~nF.log"
			IF EXIST "!LOG_PATH!" DEL "!LOG_PATH!"
		)
		echo Removed previous conversion products in GameDataFolder.
	) ELSE (
		EXIT
	)
)

IF !PRIOR_GDFX!==0 (
	IF !PRIOR_CONV!==0 (
		echo All clear.
	)
)

REM ==========================
echo Checking for game data...
SET FAILED_RETAIL_VAL=0
SET FAILED_DEMO_VAL=0
FOR %%N IN (0 1 2 3 4 6 8 9 10 11 12 13 14 18 19) DO (
	IF NOT EXIST GameDataFolder\level%%N_Final.dat (
		SET FAILED_RETAIL_VAL=1
		echo level%%N_Final.dat not found.
	)
)

IF !FAILED_RETAIL_VAL!==1 (
	REM Only our repackaged demo contains all three levels, but that's okay because it's highly
	REM unlikely that they're using the original self-destructing demo and trying to convert it
	FOR %%N IN (0 1 2 4) DO (
		IF NOT EXIST GameDataFolder\level%%N_Final.dat (
			SET FAILED_DEMO_VAL=1
			echo level%%N_Final.dat not found.
		)
	)
	
	IF !FAILED_DEMO_VAL!==1 (
		echo Some of the game data files needed for running Oni are missing. Supply a retail or
		echo demo copy of Oni. Visit http://mods.oni2.net/ and search for 'demo' to get the demo.
		PAUSE
		EXIT
	) ELSE (
		echo This is the demo installation of Oni. There's no need to convert it if this is the
		echo Windows demo, as the game data uses the same format as OniX. If this is the Mac demo,
		echo you should visit http://mods.oni2.net/, search for 'demo', and download the Windows
		echo version, which comes with OniX and is ready to play.
		PAUSE
		EXIT
	)
) ELSE (
	echo Retail data detected.
)

REM ==========================
echo Determining platform version of game data...
IF EXIST GameDataFolder\level*.sep (
	SET HAS_SEP=1
	echo Found some .sep files; assuming this is Mac data.
) ELSE (
	echo No .sep files detected; assuming this is Windows data.
)

REM ==========================
echo Ensuring completeness of game data...
FOR %%F IN (GameDataFolder\level*.dat) DO (
	IF NOT EXIST GameDataFolder\%%~nF.raw (
		SET RAWSEP_MISSING=1
		echo %%~nF.raw not found.
	)
	
	IF !HAS_SEP!==1 (
		IF NOT EXIST GameDataFolder\%%~nF.sep (
			SET RAWSEP_MISSING=1
			echo %%~nF.sep not found.
		)
	)
)
IF !RAWSEP_MISSING!==1 (
	echo GameDataFolder needs to contain a full set of game data files. The file^(s^) above are missing.
	PAUSE
	EXIT
)
echo All needed files present.

REM ==========================
echo Testing privileges...
GameDataTool -export:TXMPKONdeepthought GameDataFolder\level0_Final GameDataFolder\level0_Final.dat > nul
IF errorLevel 1 (
	echo This script does not seem to have sufficient privileges to write to GameDataFolder. Are
	echo you an administrator? Do you have ownership of the Oni folder? You could try moving the
	echo entire Oni folder to another drive, or to your desktop. If that does not work, you will
	echo need to get Properties on the Oni folder and use the Security tab to give yourself more
	echo rights to it and all subfolders.
	PAUSE
	EXIT
)
RMDIR GameDataFolder\level0_Final /q /s
echo Privileges sufficient.


REM ==========================
echo Converting the game data...
FOR %%F IN (GameDataFolder\level*.dat) DO (
	SET FNAME=%%~nF
	REM Construct a hypothetical folder name; GameDataTool will create it if it doesn't exist, as well as GDFX
	SET FOLDER="GameDataFolder\!FNAME!"

	echo Exporting vanilla-format !FNAME!...
	GameDataTool -export "!FOLDER!" GameDataFolder\!FNAME!.dat > GameDataFolder\!FNAME!.log
	IF errorlevel 1 GOTO telluser
	GameDataTool -move "!FOLDER!\SNDD" "!FOLDER!\SNDD*.oni" > nul
	IF errorlevel 1 GOTO telluser

	echo Extracting sounds...
	IF EXIST !FNAME!.sep (
		GameDataTool -extract:aif "!FOLDER!\SNDD\aif" "!FOLDER!\SNDD\SNDD*.oni" >> GameDataFolder\!FNAME!.log 2>&1
		IF errorlevel 1 GOTO telluser
		echo Reformatting sounds...
		GameDataTool -create "!FOLDER!" -demo -forcestd "!FOLDER!\SNDD\aif\SNDD*.aif" >> GameDataFolder\!FNAME!.log
		IF errorlevel 1 GOTO telluser
	) ELSE (
		GameDataTool -extract:wav "!FOLDER!\SNDD\wav" "!FOLDER!\SNDD\SNDD*.oni" >> GameDataFolder\!FNAME!.log 2>&1
		IF errorlevel 1 GOTO telluser
		echo Reformatting sounds...
		GameDataTool -create "!FOLDER!" -demo -forcestd "!FOLDER!\SNDD\wav\SNDD*.wav" >> GameDataFolder\!FNAME!.log
		IF errorlevel 1 GOTO telluser
	)
	echo Creating OniX-format !FNAME!...
	GameDataTool -import:onix "!FOLDER!" GameDataFolderX\!FNAME!.dat >> GameDataFolder\!FNAME!.log
	
	:telluser
	IF errorlevel 1 (
		echo An error was encountered while converting the game data.
		echo Look at GameDataFolder\!FNAME!.log to see the issue.
		PAUSE
		EXIT
	)
	
	REM Clean up temp folder
	RMDIR "!FOLDER!" /q /s
)

REM Copy over the rest of GDF to GDFX if not already present
XCOPY GameDataFolder\IGMD GameDataFolderX\IGMD /s /i /y
XCOPY GameDataFolder\*.bik GameDataFolderX /y

echo The conversion was completed successfully.
PAUSE
